@@ -1,15 +1,35 @@
-#######################################################################
+########################################################################
+# FindBin::libs_curr
+#
+# use $FindBin::Bin to search for 'lib' directories and use them.
+#
+# default action is to look for dir's named "lib" and silently use
+# the lib's without exporting anything. print turns on a short 
+# message with the abs_path results, export pushes out a variable
+# (default name is the base value), verbose turns on decision output
+# and print. export takes an optional argument with the name of a
+# variable to export.
+#
+# 
+########################################################################
+
+########################################################################
 # housekeeping
 ########################################################################
 
-package FindBin::libs   v2.20.2;
-use v5.28;
+package FindBin::libs;
+
+use v5.14;
+use strict;
 
 use FindBin;
+use Symbol;
+
 use File::Basename;
 
-use Carp    qw( croak                   );
-use Symbol  qw( qualify qualify_to_ref  );
+use Carp        qw( croak           );
+use List::Util  qw( first           );
+use Symbol      qw( qualify_to_ref  );
 
 use File::Spec::Functions
 qw
@@ -18,7 +38,6 @@ qw
     &splitdir
     &catpath
     &catdir
-    &canonpath
 );
 
 BEGIN
@@ -35,45 +54,43 @@ BEGIN
     # cannot handle the rooted system being linked
     # back to itself.
 
-    use Cwd qw( &abs_path &cwd );
+    use Cwd qw( &cwd );
+
+    my $abs = Cwd->can( 'abs_path'  )
+    or die "Odd: Cwd cannot 'abs_path'\n";
 
     if
     (
-        # abs_path has a fixed bug dealing with infinite
-        # recursion. if upping the version of Cwd does 
-        # not fix this then the only other test I can 
-        # think of is ( -e '/.' && -e '/..' && -e '/../.' )
-
-        eval 
-        {
-            abs_path '//';
-            abs_path cwd
-        }
+        eval { $abs->( '//' );  $abs->( cwd ); 1 }
     )
     {
-        # abs_path seems clean on this platform.
+        # nothing more to do: abs_path works.
+    }
+    elsif
+    (
+        $abs = File::Spec::Functions->can( 'rel2abs'  )
+    )
+    {
+        # ok, we have a substitute
     }
     else
     {
-        # abs_path seems to be having problems,
-        # fix is to stub it out.
-        #
-        # undef avoids nastygram.
+        die "Non-working abs_path without 'rel2abs'\n";
+    }
 
-        my $ref = qualify_to_ref 'abs_path', __PACKAGE__;
+    my $ref = *{ qualify_to_ref 'abs_path', __PACKAGE__ };
 
-        my $sub = File::Spec::Functions->can( 'rel2abs' );
+    undef &{ *$ref };
 
-        undef &{ $ref };
-
-        *$ref = $sub
-    };
+    *{ $ref } = $abs;
 }
 
 ########################################################################
 # package variables 
 ########################################################################
 
+our $VERSION = v2.0;
+
 my %defaultz = 
 (
     base    => 'lib',
@@ -83,7 +100,6 @@ my %defaultz =
     subdir  => '',      # add this subdir also if found.
     subonly => undef,   # leave out lib's, use only subdir.
     export  => undef,   # push variable into caller's space.
-    append  => undef,   # push onto existing array (vs. overwrite)
     verbose => undef,   # boolean: print inputs, results.
     debug   => undef,   # boolean: set internal breakpoints.
 
@@ -94,6 +110,8 @@ my %defaultz =
     ignore => '/,/usr', # dir's to skip looking for ./lib
 );
 
+my @use_undef = qw( export ignore );
+
 # only new directories are used, ignore pre-loads
 # this with unwanted values.
 
@@ -101,6 +119,7 @@ my %found = ();
 
 # saves passing this between import and $handle_args.
 
+my %argz    = ();
 my $verbose = '';
 my $empty   = q{};
 
@@ -116,11 +135,11 @@ my $empty   = q{};
 my $find_libs
 = sub
 {
-    my $argz    = shift;
-    my $base    = basename $argz->{ base };
-    my $subdir  = canonpath $argz->{ subdir } || '';
+    my $base    = basename ( shift || $argz{ base } );
 
-    my $subonly = defined $argz->{ subonly };
+    my $subdir  = $argz{ subdir } || '';
+
+    my $subonly = defined $argz{ subonly };
 
     # for some reason, RH Enterprise V/4 has a 
     # trailing '/'; I havn't seen another copy of 
@@ -134,7 +153,7 @@ my $find_libs
     # after that splitpath can grab the directory 
     # portion for future use.
 
-    my ( $Bin ) = ( $argz->{ Bin } =~ m{^ (.+) }xs );
+    my ( $Bin ) = ( $argz{ Bin } =~ m{^ (.+) }xs );
 
     print STDERR "\nSearching $Bin for '$base'...\n"
     if $verbose;
@@ -187,7 +206,7 @@ my $find_libs
 
                 push @libz, $dir;
 
-                last if $argz->{ scalar };
+                last PATH if $argz{ scalar };
             }
         }
 
@@ -198,13 +217,12 @@ my $find_libs
     # (including volume) walking up the path 
     # from $FindBin::Bin -> root.
     #
-    # no libs found is empty list or undef for 
-    # scalar.
-    #
     # passing it back as a list isn't all that
     # painful for a few paths.
 
-    wantarray ? @libz : \@libz
+    wantarray 
+    ?  @libz 
+    : \@libz
 };
 
 # break out the messy part into a separate block.
@@ -219,26 +237,21 @@ my $handle_args
     # anything after the module are options with arguments
     # assigned via '='.
 
-    my %argz
+    %argz
     = map
     {
-        my $use_undef
-        = do
-        {
-            my %a   = ();
-            @a{ qw( export ignore ) } = ();
-            \%a
-        };
-
         my ( $k, $v ) = split '=', $_, 2;
 
-        exists $use_undef->{ $k }
-        or $v //= 1;
+        defined $v
+        or 
+        first { $_ eq $k } @use_undef 
+        or 
+        $v  //= 1;
 
         # "no" inverts the sense of the test.
 
-        $k =~ s{^no}{}
-        and $v  = ! $v;
+        index $k, 'no'
+        or $v  = ! $v;
 
         ( $k => $v )
     }
@@ -288,7 +301,7 @@ my $handle_args
     }
 
     exists $argz{ base } && $argz{ base } 
-    or croak "Bogus FindBin::libs: missing/false base argument, should be 'base=NAME'";
+    or croak "Bogus FindBin::libs: false base argument, should be 'base=NAME'";
 
     exists $argz{ export }
     and
@@ -322,27 +335,22 @@ my $handle_args
             }
         }
     }
-
-    %argz
 };
 
 sub import
 {
-    # map used here for -T checks. everything from find_libs has
-    # already been through abs_path, not much more to check here.
+    &$handle_args;
 
-    my %argz    = &$handle_args;
-    my @libz    
-    = map
-    {
-        my ( $path ) = m{ (.+) }x; 
+    my @libz = $find_libs->();
 
-        ( $path =~ /\S/ )
-        ? $path
-        : ()
-    }
-    $find_libs->( \%argz )
-    ;
+    # HAK ALERT: the regex does nothing for security,
+    # just dodges -T. putting this down here instead
+    # of inside find_libs allows people to use saner
+    # untainting plans via find_libs.
+    #
+    # "s" allows directories to contain newlines. 
+
+    @libz   = map { m{ (.+) }xs } @libz;
 
     my $caller = caller;
 
@@ -357,77 +365,60 @@ sub import
 
     if( $argz{ export } )
     {
-        # this has to run in order to install variables that
-        # the caller is expecting to exist at runtime -- even
-        # if they are empty/undef at the end of it.
-
-        my $ref = qualify_to_ref $argz{ export }, $caller;
+        my $dest    = qualify        $argz{ export }, $caller;
+        my $ref     = qualify_to_ref $dest;
 
         if( $verbose )
         {
-            my $dest    = qualify $argz{ export }, $caller;
-
             $argz{ scalar }
             ? print STDERR "\nExporting: \$$dest\n"
             : print STDERR "\nExporting: \@$dest\n"
-            ;
         }
 
+        # Symbol this is cleaner than "no strict" 
+        # for installing the array.
+
         if( $argz{ scalar } )
         {
-            *$ref 
-            = @libz
-            ? \$libz[0]
-            : \( my $a = '' )
-            ;
-        }
-        elsif
-        (
-            $argz{ append } 
-            and
-            my $ary = *{ $ref }{ ARRAY }
-        )
-        {
-            push @$ary, @libz;
+            *$ref = \$libz[0];
         }
         else
         {
-            *$ref = \@libz
+            *$ref = \@libz;
         }
     }
 
-    if( @libz )
+    if( defined $argz{ p5lib } )
     {
-        # no 'else', these are not exclusive
+        # stuff the lib's found at the front of $ENV{ PERL5LIB }
 
-        if( defined $argz{ p5lib } )
-        {
-            # stuff the lib's found at the front of $ENV{ PERL5LIB }
-            # yes, virginia, substr is an lvalue -- and saner than
-            # dealing with \Q and a regex on arbitrary paths.
+        ( substr $ENV{ PERL5LIB }, 0, 0 ) = join ':', @libz, ''
+        if @libz;
 
-            ( substr $ENV{ PERL5LIB }, 0, 0 ) = join ':', @libz, '';
+        print STDERR "\nUpdated PERL5LIB:\t$ENV{ PERL5LIB }\n"
+        if $verbose;
+    }
 
-            print STDERR "\nUpdated PERL5LIB:\t$ENV{ PERL5LIB }\n"
-            if $verbose;
-        }
+    if( $argz{ use } && @libz )
+    {
+        # this obviously won't work if lib ever depends 
+        # on the caller's package.
+        #
+        # it does avoids issues with -T blowing up on the
+        # old eval technique.
 
-        if( $argz{ use } )
-        {
-            # this obviously won't work if lib ever depends 
-            # on the caller's package.
+        require lib;
 
-            require lib;
-
-            lib->import( @libz );
-        }
+        lib->import( @libz );
     }
 
     0
 }
 
 # keep require happy
+
 1
+
 __END__
 
 =head1 NAME
@@ -437,6 +428,9 @@ directories based on $FindBin::Bin.
 
 =head1 SYNOPSIS
 
+This version of FindBin::libs is suitable for 
+Perl v5.10+.
+
     # search up $FindBin::Bin looking for ./lib directories
     # and "use lib" them.
 
@@ -527,6 +521,11 @@ directories based on $FindBin::Bin.
     use FindBin::libs qw( export=junk  base=frobnicatorium                  );
     use FindBin::libs qw( export       base=foobar                          );
 
+    # avoid dealing with an array by taking the first one found.
+    # in this case $etc (rather than @etc).
+
+    use FindBin::libs qw( export=etc scalar base=etc ).
+
 =head1 DESCRIPTION
 
 =head2 General Use
@@ -560,24 +559,12 @@ that
 
 will search for directories named "altlib" and "use lib" them.
 
-=head3 Exporting a variable: "export", "scalar", "append"
+=head3 Exporting a variable: 'export'
 
-=over 4
-
-=item "export"
-
-This installs the results of locating directories into the caller's 
-space. Without any argument, export pushes out a variable named after 
-the located [sub]dir; an argument can be supplied to give the variable 
-name. Without the "scalar" option, the exported variable will be an 
-array in increasing order of "distance" (i.e., "up" the file tree); 
-with the "scalar" option only the first (i.e., "nearest") path is 
-exported.
-
-If "export" is given then "nouse" is assumed; using both leaves the 
-variable exported and its contents handed to "use lib".
-
-For example:
+The 'export' option will usually (see "scalar", below) 
+push an array of the directories found and takes an 
+optional argument of the array name, which defaults to the 
+basename searched for:
 
     use FindBin::libs qw( export );
 
@@ -607,7 +594,7 @@ exports @meta while
 
     use FindBin::libs qw( export=metadirs base=meta );
 
-exports @metadirs as a list of paths ending in "/meta".
+exports @metadirs.
 
 The use and export switches are not exclusive:
 
@@ -616,85 +603,33 @@ The use and export switches are not exclusive:
 will locate "lib" directories, use lib them, and export 
 @mylibs into the caller's package. 
 
-=item "scalar"
+There are times when looking down multiple entries is annoying
+becuase only the first one is useful. In this case the "scalar"
+argument reduces an array to the first value. The most common
+example of this is looking for configuration files in the 
+"nearest" ./etc directory.
 
-Only searches for the first directory, which is exported (or
-overwritten) as a scalar rather than array. For example, if
-a project directory has ./bin and ./etc dir's then #! code in
-bin with
+For example:
 
-    use FindBin::libs qw( export scalar base=etc );
+    use FindBin::libs qw( base=etc export=config scalar ); 
 
-will have an $etc variable with the absolute path to ./bin/../etc.
-For configuration varibles this is usually what you want and allows
-for "$etc/Foo.conf" rather than "$etc[0]/Foo.conf".
-
-=item "append"
-
-Sometimes it's simpler to accumulate multiple searches into a 
-single array. Say for ./etc dir's in collection of standard
-locations.
-
-In that case:
-
-    use FindBin::libs qw( export=etc base=foo subdir=etc );
-    use FindBin::libs qw( export=etc base=bar subdir=etc append );
-
-produces something like
-
-    (
-        /path/to/foo/etc
-        /path/to/bar/etc
-    )
-
-without append @etc will have only ./bar/etc since the array would
-be overwritten with each call to import.
-
-=back
+will export the first ./etc path it finds into $config.
 
 =head3 Subdirectories
 
 The "subdir" and "subonly" settings will add or 
 exclusively use subdir's. This is useful if some
 of your lib's are in ../lib/perl5 along with 
-../lib or all of the lib's are in ../lib/perl5.
-
-These could be handled with:
-
-    use FindBin::libs;
-    use FindBin::libs qw( subdir=perl5 subonly );
-
-which uses the "lib" dir's along with any lib/perl5 dirs.
+../lib (subdir=perl5) or all of the lib's are 
+in ../lib/perl5 (subonly=perl5).
 
 This can also be handy for locating subdir's used
 for configuring packages:
 
     use FindBin::libs qw( export base=config subonly=mypackage );
 
-Will leave @config containing any mypackage dir's found up
-the tree, nearest to closest.
-
-The array format is convienent for locating configuration files
-shared between projects in separate, sibling directories. For
-example given:
-
-    ./proj/Foo/etc
-    ./proj/etc
-
-with
-
-    use FindBin::libs qw( export subdir=etc subonly )
-
-will export @etc with qw( ../proj/Foo/etc ../proj/etc ) in lexical
-order by distance from the #! code. At that point
-
-    use List::Util qw( first );
-
-    my $path = first { -e "$_/Global.config" } @etc;
-
-will locate the nearest "Global.confg" file. Note that this is 
-not the same as using "scalar" since that will export 
-$etc with only ./Foo/etc. 
+Will leave @config with any "mypackage" holding
+any "mypackage" subdir's.
 
 =head3 Setting PERL5LIB: p5lib
 
@@ -725,6 +660,8 @@ to something like:
 This can make controlling the paths used simpler and avoid
 the use of symlinks for some testing (see examples below).
 
+Note that "p5lib" and "nouse" are proably worth 
+
 =head2 Skipping directories
 
 By default, lib directories under / and /usr are
@@ -772,7 +709,7 @@ having to modify a single line of code.
 Say your sandbox is in ./sandbox and you are currently
 working in ./sandbox/projects/package/bin on a perl
 executable. You may have some number of modules that
-are specific -- or customized -- for this pacakge, 
+are specific -- or customized -- for this package, 
 share some modules within the project, and may want 
 to use company-wide modules that are managed out of 
 ./sandbox in development. All of this lives under a 
@@ -929,7 +866,7 @@ may be limited to *NIX or at least less portable.
 
 PERL5LIB can be used to accomplish the same directory
 lookups as FindBin::libs.  The problem is PERL5LIB often
-contains absolute paths and does not automatically change
+contains absolte paths and does not automatically change
 depending on where tests are run. This can leave you 
 modifying a file, changing directory to see if it works
 with some other code and testing an unmodified version of 
@@ -971,7 +908,7 @@ in the executable path (or cwd) and uses that.
 
 The main problem here is that if the anchor ever changes
 (e.g., when moving code between projects or relocating 
-directories) the path often has
+directories now that SVN supports it) the path often has
 to change in multiple files. The regex also may have to
 support multiple platforms, or be broken into more complicated
 File::Spec code that probably looks pretty much like what
@@ -1036,12 +973,8 @@ directory paths based on $FindBin::Bin.
 
 =item Older code.
 
-There are versions of the code for Perl from releases
-v5.8, 5.14, and 5.28. Makefile.PL will pick the most recent
-version it can based on $^V at Configure time. As of this
-writing Perl is at 5.32, which would pick the 5.28 release. 
-
-The version-specific code is in ./version/<perl-v-string.
+FindBin::libs_5_8.pm is installed if $^V indicates
+that the running perl is prior to v5.10.
 
 =back
 
@@ -1072,12 +1005,12 @@ working version) once the File::Spec is fixed.
 
 =item 
 
-The hack for older versions of perl is 
+The hack for prior-to-5.12 versions of perl is 
 messy, but is the only I've found that works for
 the moment on *NIX, VMS, and MSW. I am not sure
 whether any of these systems are normally configured
 to share perl modules between versions. If the 
-modules are not shared on multiple platforms then
+moduels are not shared on multiple platforms then
 I can make this work by managing the installation
 rather than checking this every time at startup.
 
@@ -1085,28 +1018,12 @@ For the moment, at least, this seems to work.
 
 =back
 
-=head1 SEE ALSO
-
-=over 4
-
-=head2 Cwd
-
-A bug in Cwd was fixed with 3.73. That should, hopefully,
-fix the issue with FB::l croaking perl during testing.
-If it does not then I will have to modify the sanity check
-for using abs_path vs. rel2abs.
-
 =head1 AUTHOR
 
 Steven Lembark, Workhorse Computing <lembark@wrkhors.com>
 
 =head1 COPYRIGHT
 
-Copyright (C) 2003-2020, Steven Lembark, Workhorse Computing.
-This code is released under the same terms as Perl-5.28
-or any later version of Perl.
-
-=head1 LICENSE
-
-This code is released under the same terms as Perl-5.28
+Copyright (C) 2003-2012, Steven Lembark, Workhorse Computing.
+This code is released under the same terms as Perl-5.10
 or any later version of Perl.
